import sys, os
import numpy as np
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt

import BioNetTopology, BioNetMeasurement

sys.setrecursionlimit(100000)

def recursivelyAddNodesToCluster(index, cluster, remaining, nConnections):

	# Have we been here?
	if(index not in cluster):

		# Add to cluster
		cluster.add(index)

		# Remove from global list
		remaining.remove(index)

		# Get next node
		for nIndex in nConnections[index]:
			cluster, remaining = recursivelyAddNodesToCluster(nIndex, cluster, remaining, nConnections)

	return cluster, remaining

#
# Graph shit
#

SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#

# Constants
numNodes = 5000

# Variables
numRuns = 3
#ffs = [0.72,0.91, 1.0]
ffs = [0.72, 0.91, 1.0]

pbcStr = ""

# Loop over variables
for ff in ffs:
	for nr in range(numRuns):

		# Get the topology
		tpFname = "../output/BSAHydrogel_ff%.3f_Kinetics_Run%d%s.bntop" % (ff,nr, pbcStr)
		if(not os.path.exists(tpFname)):
			continue

#		top = BioNetTopology.BioNetTopology()
#		top.loadFinalFrame(tpFname)

		top = BioNetTopology.BioNetTopology()
		top.load(tpFname, rate=10)

		tpFname = "../output/BSAHydrogel_ff%.3f_KineticsRestart_Run%d%s.bntop" % (ff,nr, pbcStr)
		if(not os.path.exists(tpFname)):
			continue

#		top2 = BioNetTopology.BioNetTopology()
#		top2.loadFinalFrame(tpFname)

		top2 = BioNetTopology.BioNetTopology()
		top2.load(tpFname, rate=10)

		mFname = "../output/BSAHydrogel_ff%.3f_Kinetics_Run%d.bnms" % (ff,nr)
		meas = BioNetMeasurement.BioNetMeasurement()
		meas.load(mFname, rate=10)

		mFname = "../output/BSAHydrogel_ff%.3f_KineticsRestart_Run%d.bnms" % (ff,nr)
		meas2 = BioNetMeasurement.BioNetMeasurement()
		meas2.load(mFname, rate=10)

		# Combine as necessary
		newTime = meas2.time + meas.time[-1]
		meas.time = np.append(meas.time, newTime[1:])
		top.numFrames += top2.numFrames - 1
		top.frame.extend(top2.frame[1:])

		# Find the time
		end = 0
		lim = 5000
		while(True):
			if(end == meas.time.size or meas.time[end] > lim):
				end -= 1
				break
			end += 1

		top.frame = top.frame[:end]

		# Connect all the nodes
		top.frame[-1].buildNodeConnections(top.numNodes)

		# Get a set of all node indices
		nodesRemaining = set(range(top.numNodes))

		# Initialise clusters
		clusters = []

		# Search while there are still nodes unaccounted for
		while(len(nodesRemaining) != 0):

			# New cluster
			cluster = set([])

			# Get a central node
			centralNode = nodesRemaining.pop()

			# Readd for recursion
			nodesRemaining.add(centralNode)

			# Recursively add
			cluster, nodesRemaining = recursivelyAddNodesToCluster(centralNode, cluster, nodesRemaining, top.frame[-1].nConnections)
			
			# Add to clusters
			clusters.append(cluster)

		# Info
		numClusters = len(clusters)
		lengths = [len(c) for c in clusters]
		minCluster = min(lengths)
		maxCluster = max(lengths)

		print("FF: %.2f " % (ff))
		print("Num Clusters: %d, Max: %d, Min: %d\n" % (numClusters, maxCluster, minCluster))

		# Write clusters to file
		oFname = "../results/BSAHydrogel_ff%.3f_Kinetics_Run%d%s_Clusters.dat" % (ff,nr, pbcStr)
		fout = open(oFname, "w")
		fout.write("numClusters %d\n" % (len(clusters)))
		for cluster in clusters:
			c = list(cluster)
			fout.write("%d" % (c[0]))
			for i in c[1:]:
				fout.write(",%d" % (i))
			fout.write("\n")
		fout.close()
