#import sys, os
import copy
import numpy as np
from matplotlib import pyplot as plt

import BioNetInput, BioNetTrajectory, BioNetMeasurement


#
# Graph shit
#
SMALL = 8
MEDIUM = 10
LARGE = 12

#
# Data
#
numRuns = 3
#ffs = [0.72,0.91, 1.0]
ffs = [0.72,0.91,1.0]

# Other data
numNodes = 5000
vf = 0.074
radius = 2.5
boxDims = np.power((4.0/3.0) * np.pi * numNodes / vf,0.3333) * radius
ln = 10.7368
kbt = 4.11
drag = 6 * np.pi * radius
kCross = 100

# Loop over variables (including numRuns)
fig = plt.figure(figsize=(3.2,3.2))

# BC
pbcStr = ""

fIndex = 0
for ff in ffs:

	# Data
	dFname = "../results/BSAHydrogel_ff%.3f_FractalDimensions" % (ff) + pbcStr + ".dat"

	fout = open(dFname, "w")
	fout.write("x0, x0Err, m1, m1Err, x01, x01Err, FD, FDErr\n")

	# Averages
	fds = []
	fderrs = []

	for nr in range(numRuns):

		# Get initial script
		iFname = "../input/BSAHydrogel_ff%.3f_Kinetics_Run%d.xml" % (ff,nr)
		inp = BioNetInput.BioNetInput()
		inp.load(iFname)


		# Get trajectories
		tFname = "../output/BSAHydrogel_ff%.3f_Kinetics_Run%d.bntrj" % (ff,nr)
		traj = BioNetTrajectory.BioNetTrajectory()
		traj.loadFinalFrame(tFname)


		mFname = "../output/BSAHydrogel_ff%.3f_Kinetics_Run%d.bnms" % (ff,nr)
		meas = BioNetMeasurement.BioNetMeasurement()
		meas.load(mFname, rate=10)

		# PBC
		if pbcStr !="_HBC":
			periodic = True
		else:
			periodic = False

		# Get radii
		rads = np.array([n.protein.radius for n in inp.node])

		# Set a threshold
		thresh = 2.5
		fitFuncIndex = 2

		# Calc FD
		fdPair, data = traj.frame[-1].calculateFractalDimension_BoxCount(inp.box.dimension, thresh, fitFuncIndex, radius = rads, periodic=periodic)

		# Write
		popt = data[3]
		perr = np.sqrt(np.diag(data[4]))

#			fout.write("%f,%f,%f,%f" % (fdPair[0], fdPair[1], data[3][0], np.sqrt(data[4][0][0])))
		
		fout.write("%f,%f" % (popt[0], perr[0]) )
		for z in range(3):
			fout.write(",%f,%f" % (popt[z+1], perr[z+1]) )
		fout.write("\n")

		# Add to list
		fds.append(fdPair[0])
		fderrs.append(fdPair[1])

		# Plot everything (fit and all) (we haven't taken averages here)
		if(nr == 0):
			XData = data[0]
			YData = data[1]
			fitFunc = data[2]
			popt = data[3]
			pcov = data[4]

			plt.plot(XData, YData, "oC%d" % (fIndex), ms=4, label="ff = %.3f" % (ff))
			plt.plot(XData, fitFunc(XData, *popt), "--C%d" % (fIndex))

	fd = np.mean(fds)
	fderr = np.mean(fderrs) + np.std(fds) / np.sqrt(numRuns)

	fout.write("%f,%f\n" % (fd, fderr))

	print("ff %.3f: Fractal Dimension = %f +/- %f" % (ff, fd,fderr))
	fout.close()
	fIndex += 1

# Prettify
plt.xlabel(r"log $_2$ (Num Boxes 1D)", fontsize=MEDIUM)
plt.ylabel(r"log $_2$ (NumBoxes Populated)", fontsize=MEDIUM)
plt.legend(loc=0, fontsize=SMALL)
plt.xlim(plt.xlim()[0], 7.5)
plt.ylim(plt.ylim()[0], 20)
plt.tight_layout()
oFname2 = "../results/BSAHydrogel_Kinetics_Run%d" % (0) + pbcStr + "_BoxCount.png"
plt.savefig(oFname2, dpi=100)
plt.clf()
