import sys, os
import numpy as np

import BioNetInput, BioNetTopology

#
# Data
#
numRuns = 3
#ffs = [0.72,0.91, 1.0]
ffs = [0.72, 0.91, 1.0]

# Other data
numNodes = 5000
vf = 0.074
radius = 2.5
boxDims = np.power((4.0/3.0) * np.pi * numNodes / vf,0.3333) * radius
ln = 10.7368
kbt = 4.11
drag = 6 * np.pi * radius
kCross = 100

# BC
pbcStr = ""

fIndex = 0
for ff in ffs:

	for nr in range(numRuns):

		# Get initial script
		iFname = "../input/BSAHydrogel_ff%.3f_Kinetics_Run%d" % (ff,nr) + pbcStr + ".xml"

		inp = BioNetInput.BioNetInput()
		inp.load(iFname)

		# Get final topology frame
		tpFname = "../output/BSAHydrogel_ff%.3f_Kinetics_Run%d" % (ff,nr) + pbcStr + ".bntop"

		top = BioNetTopology.BioNetTopology()
		top.loadFinalFrame(tpFname)

		# Loop over all connections and verify that the only AA bonds are AALinker bonds, not cross-links
		problemAAAACrosslinks = 0
		problemBSAAACrosslinks = 0
		problemLinkers = 0
		goodLinkers = 0
		goodCrosslinks = 0
		for c , b in zip(top.frame[-1].cConnections, top.frame[-1].bIndex):

			nIndex = [c[0][1], c[1][1]]
			if inp.node[nIndex[0]].protein.name == "BSA" and inp.node[nIndex[1]].protein.name == "BSA":
				if inp.bond[b].name != "Crosslink":
					problemLinkers += 1
				else:
					goodCrosslinks += 1

			elif inp.node[nIndex[0]].protein.name == "AminoAcidNode" and inp.node[nIndex[1]].protein.name == "AminoAcidNode":
				if inp.bond[b].name == "Crosslink":
					problemAAAACrosslinks  += 1
				else:
					goodLinkers += 1

			else:
				problemBSAAACrosslinks += 1

		print("ff: %.3f, Run %d\n" % (ff, nr))
		print("AA-AA Linkers = %d" % (goodLinkers))
		print("BSA-AA Linkers = %d\n" % (problemLinkers))
		print("AA-AA Crosslinks = %d" % (problemAAAACrosslinks))
		print("BSA-AA Crosslinks = %d" % (problemBSAAACrosslinks))
		print("BSA-BSA Crosslinks = %d" % (goodCrosslinks))
		print("\n")

# self.cConnections[i][j][0] - Connection i, vertex j, child, self.cConnections[i][j][1] - Connection i, vertex j, parent


